// Rextended UT: A mutator designed to extend UT by adding more features
// Current Features: Dropped flag countdowns, Inventory counters for UT_Invisibility, UT_JumpBoots and DamageAmp, Display the name of flag carrriers
// Features continued: prevents multiple pickups
// New in version 2-5: Fixes alignment issues with flag carrier names & udamage counter, fixes udamage counter
// New Continued: Adds compatibility with Bombing Run: Shows the bomb-carrier's name on the HUD

// By <GF>-REX!!		www.UnrealAddiction.com
class REXUT extends Mutator config(RextendedUT);

#exec TEXTURE IMPORT NAME=RBootsIcon FILE=Textures\RBootsIcon.pcx GROUP=Icons
#exec TEXTURE IMPORT NAME=RInvisIcon FILE=Textures\RInvisIcon.pcx GROUP=Icons

var config bool bShowFlagCountDown, bShowFlagCarrierName, bShowAmpCountdown, bShowInvisCountdown, bShowBootCharge;
var config bool bPreventArmorPickupWhenFull, bAlwaysPreventMultiBeltPickup, bPreventMultiDamAmp, bPreventMultiBootsWhenFull, bPreventMultiInvis;
var config bool bPreventArmorPickupWhenHasFullBelt, bPreventArmorPickupWhenHasDamagedBelt; 
var config bool bPreventMultiScubaGear, bPreventMultiFlashLight, bPreventMultiSearchLight; // prevent multi-pickups
var config bool bPreventMultiSuits, bPreventAllMultiPickups; // prevent multi-pickups
var RexFlagWatcher FlagRed, FlagBlue; // Objects used to store the timer for each flag
var int CountdownR, CountdownB; // Dropped Flag countdowns
var bool bRedOut, bBlueOut; // Which flag is out
var PlayerReplicationInfo FlagCarrier[2], BombHolder; // Who has the flag? 0=red 1=blue, BombHolder for Bombing Run
var config bool bModifyTriggers; // Do you want to modify any triggers?
var config  bool bShowBombingRunCarrier;
var config string MapToModify[30]; // Triggers to modify (Changes ReTriggerDelay) along with what maps to do it on
var config name ModifiedTriggerEvent[30]; // Used to locate the trigger to modify
var config int NewReTriggerDelay[30]; // What do you want us to change it to?
var Decoration BombingBall;
var RexBRNotify BombWatcher;


replication
{
	// Things the server should send to the client.
	reliable if ( Role == ROLE_Authority )
		bRedOut, bBlueOut, CountdownR, CountdownB, FlagCarrier, BombHolder; //bRunningUU, UUHUD, 
}

function PostBeginPlay()
{
	local Trigger t;
	local int i;
	local object B;

	Enable('Tick');
	if ( bShowFlagCountDown )
	{ // Set things up
		FlagRed = spawn(class'RexFlagWatcher');
		FlagBlue = spawn(class'RexFlagWatcher');
		FlagRed.SetOwner(self);
		FlagRed.bRed = True;
		FlagBlue.SetOwner(self);
		FlagBlue.bRed = False;
	}
	if ( bModifyTriggers )
	{
		while ( i < 30 )
		{
			if ( GetLevelName() ~= MapToModify[i] )
			{
				ForEach AllActors(class'Trigger', t )
				{
					if ( t.Event == ModifiedTriggerEvent[i] )
					{
						Log("RextendedUT: We modified triggers with event:"@t.Event@"on"@GetLevelName());
						t.ReTriggerDelay = NewReTriggerDelay[i];
					}
				}
				i = 52;
			}
			i++;
		}
	}
		
	Level.Game.RegisterMessageMutator(Self);
	if ( !Level.Game.IsA('CTFGame') )
	{// Automatically disable CTF features if the GameType is not CTF
		Log("RexUT: We are not in CTF mode, so we will stop listening for messages");
		//Disable('MutatorBroadcastLocalizedMessage');
		FlagRed.Destroy();
		FlagBlue.Destroy();
		Log("RextendedCTF: GameType is not CTF, so CTF flag countdowns are disabled!");
		if ( Level.Game.IsA('aBombingRun') )
		{
			if ( bShowBombingRunCarrier )
			{
				BombHolder = None;
				BombWatcher = Spawn(class'RexBRNotify');
				BombWatcher.MasterRex = Self;
				Log("RextendedBR: GameType is BR and we are ready to show bomb carriers");
			}
		}
	}
	SaveConfig(); //Makes sure there is an INI file
	if ( !Level.Game.IsA('aBombingRun') )
	{
		bShowBombingRunCarrier = false;
	}
	Super.PostBeginPlay();
}

simulated function string GetLevelName()
{
	local string Str;
	local int Pos;

	Str=string(Level);
	Pos = InStr(Str, ".");
	if(Pos != -1)
		return Left(Str, Pos);
	else
		return Str;
}

simulated Function Tick(float delta)
{// Makes sure we get registered (Thanks to the UnrealWiki)
	//local Decoration Ball;
	local Pawn P;

	if (Level.NetMode == NM_DedicatedServer || bHUDMutator)
	{
		//Disable('Tick');
	}
	else
	{
		RegisterHUDMutator();
	}
	
	if ( bShowBombingRunCarrier )
	{
		if ( BombHolder == None && BombingBall != None )
		{
			if ( BombingBall.Owner != None )
			{
				if ( BombingBall.Owner.IsA('Pawn') && Pawn(BombingBall.Owner).PlayerReplicationInfo != None )
				{
					BombHolder = Pawn(BombingBall.Owner).PlayerReplicationInfo;
				}
			}
		}
		else if ( BombingBall != None && BombingBall.Owner == None )
		{
			BombHolder = None;
		}
	}
}

simulated function PostRender(canvas Canvas)
{ // Display the countdowns when needed
	local float Scale;
	local int X,Y;
	local PlayerPawn PP;
	local Inventory i;
	local int ExtraR, ExtraB;
	local string FManR, FManB;

	PP = Canvas.Viewport.Actor;
	Canvas.bCenter = True;
	Scale = ChallengeHUD(Canvas.Viewport.Actor.myHUD).Scale;
	X = Canvas.ClipX - 63 * Scale;
	Y = Canvas.ClipY - 322 * Scale;
	Canvas.Font = Canvas.MedFont;
	Canvas.Style = ERenderStyle.STY_Masked;
	Canvas.DrawColor.R = 255;
	Canvas.DrawColor.G = 255;
	Canvas.DrawColor.B = 255;
	if ( bBlueOut )
	{// Blue flag is dropped
		Canvas.Font = ChallengeHUD(Canvas.Viewport.Actor.myHUD).MyFonts.GetMediumFont(516);
		Canvas.SetPos(X,Y - 155 * Scale);
		Canvas.DrawText(CountdownB,true);
	}
	if ( bRedOut )
	{// Red flag is dropped
		Canvas.Font = ChallengeHUD(Canvas.Viewport.Actor.myHUD).MyFonts.GetMediumFont(516);
		Canvas.SetPos(X,Y - (5 * Scale));
		Canvas.DrawText(CountdownR);
	}
	Canvas.Style = ERenderStyle.STY_Translucent;
	Canvas.Font = Canvas.BigFont;
	Canvas.bCenter = False;
	if ( bShowAmpCountdown || bShowInvisCountdown || bShowBootCharge)
	{// Makes sure theres a reason for checking inventory
		for ( I=PP.Inventory; I!=None; I=I.Inventory )
		{// Loops through current player's inventory
			if ( bShowInvisCountdown && UT_Invisibility(i) != None )
			{// Found a UT_Invisibility
				Canvas.SetPos(X-1220 * Scale,Y - 35 * Scale);
				Canvas.DrawIcon(texture'RInvisIcon',Scale*0.5);
				Canvas.SetPos(X-1165 * Scale,Y - 15 * Scale);
				if ( i.Charge < 1000 )
				{// charge is 3 digits or less, so print it
					Canvas.DrawText(i.Charge);
				}
				else
				{// charge is 4 digits or more, so cut it off
					Canvas.DrawText("999...");
				}
			}
			if ( bShowBootCharge )
			{// We support both the old Unreal boots and the UT boots!
				if ( I.IsA('JumpBoots') )
				{// Found the Unreal JumpBoots
					Canvas.SetPos(X-1220 * Scale,Y + 60 * Scale);
					Canvas.DrawIcon(texture'RBootsIcon',Scale*0.5);
					Canvas.SetPos(X-1165 * Scale,Y + 70 * Scale);
					if ( i.Charge < 1000 )
					{// charge is 3 digits or less, so print it
						Canvas.DrawText(i.Charge);
					}
					else
					{// charge is 4 digits or more, so cut it off
						Canvas.DrawText("999...");
					}
				}
				else if ( I.IsA('UT_JumpBoots') )
				{// Found the UT_JumpBoots
					Canvas.SetPos(X-1220 * Scale,Y + 100 * Scale);
					Canvas.DrawIcon(texture'RBootsIcon',Scale*0.5);
					Canvas.SetPos(X-1165 * Scale,Y + 120 * Scale);
					if ( i.Charge < 1000 )
					{// charge is 3 digits or less, so print it
						Canvas.DrawText(i.Charge);
					}
					else
					{// charge is 4 digits or more, so cut it off
						Canvas.DrawText("999...");
					}
				}
			}
			if ( bShowAmpCountdown && RexAmpChecker(i) != None && RexAmpChecker(i).bHasAmp)
			{// InvisChecker is misleading, it is actually used for the damage amp, but I never renamed it. We tell it to PostRender()
				RexAmpChecker(i).PostRender(Canvas);
			}
		}
	}
	Canvas.Font = ChallengeHUD(PP.MyHud).MyFonts.GetBigFont(Canvas.ClipX-500);
	if ( FlagCarrier[0] != None ) // and FindInventoryType(abrgun);
	{// Someone is holding the red flag
		if ( Len(FlagCarrier[0].PlayerName) >= 10 )
		{// length of name is over 10 chars, so shorten it
			FManR = Left(FlagCarrier[0].PlayerName,9)$"...";
			ExtraR = (10-len(FManR))*Scale; // 20
		}
		else
		{// name is less than 10 chars, so bring it closer to the flag icon
			FManR = FlagCarrier[0].PlayerName;
			ExtraR = (10-Len(FlagCarrier[0].PlayerName))*10;
		}
		Canvas.SetPos(X - (110-ExtraR) * Scale,Y + 20 * Scale);
		Canvas.DrawColor.R = 0;
		Canvas.DrawColor.G = 128;
		Canvas.DrawColor.B = 255;
		Canvas.DrawText(FManR);
	}
	if ( FlagCarrier[1] != None )// and FindInventoryType(abrgun);
	{// The blue flag is being held by somebody
		if ( Len(FlagCarrier[1].PlayerName) >= 10 )
		{// length of name is over 10 chars, so shorten it
			FManB = Left(FlagCarrier[1].PlayerName,9)$"...";
			ExtraB = (10-len(FManB))*Scale; // 20
		}
		else
		{// name is less than 10 chars, so bring it closer to the flag icon
			FManB = FlagCarrier[1].PlayerName;
			ExtraB = (10-Len(FlagCarrier[1].PlayerName))*10;
		}
		Canvas.SetPos(X - (110-ExtraB) * Scale,Y - 120 * Scale);
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 0;
		Canvas.DrawColor.B = 0;
		Canvas.DrawText(FManB);
	}
	if ( bShowBombingRunCarrier )
	{
		if ( BombHolder != None )// and FindInventoryType(abrgun);
		{// The bomb is being held by somebody

			if ( Len(BombHolder.PlayerName) >= 10 )
			{// length of name is over 10 chars, so shorten it
				FManB = Left(BombHolder.PlayerName,9)$"...";
				ExtraB = (10-Len(FManB))*Scale; // 0
			}
			else
			{// name is less than 10 chars, so bring it closer to the BR icon
				FManB = BombHolder.PlayerName;
				ExtraB = (10-Len(BombHolder.PlayerName))*10;
			}
			if ( BombHolder.Team == 0 )
			{// Red player has ball
				Canvas.SetPos(X - (110-ExtraB) * Scale,Y + 30 * Scale); //+20
				Canvas.DrawColor.R = 255;
				Canvas.DrawColor.G = 0;
				Canvas.DrawColor.B = 0;
			}
			else
			{// Blue player has ball
				Canvas.SetPos(X - (110-ExtraB) * Scale,Y - 120 * Scale);
				Canvas.DrawColor.R = 0;
				Canvas.DrawColor.G = 128;
				Canvas.DrawColor.B = 255;
			}
			Canvas.DrawText(FManB);
		}
	}
    // This is important! It allows the next HUD mutator to draw on the canvas. (Thank you UnrealWiki)
    if ( NextHUDMutator != None )
        NextHUDMutator.PostRender(Canvas);
}
function ModifyPlayer(Pawn Other)
{// Adds items to the player's inventory
	local RexAmpChecker Checker;

	if ( bShowAmpCountdown )
	{// Gives player the InvisChecker which really is a Damage Amp checker that I never renamed...
		Checker = Spawn(class'RexAmpChecker', Other,, Other.Location);
		Checker.RespawnTime = 0.0;
		Checker.GiveTo(Other);
	}
	Super.ModifyPlayer(Other);
}
function bool MutatorBroadcastLocalizedMessage( Actor Sender, Pawn Receiver, out class<LocalMessage> Message, out optional int Switch, out optional PlayerReplicationInfo RelatedPRI_1, out optional PlayerReplicationInfo RelatedPRI_2, out optional Object OptionalObject )
{// Game sends messages through here and we check if they are a CTFMessage
	// For more details check the CTFMessage class under Info>LocalMessage>LocalMessagePlusCriticalEventPlus
	local CTFFlag Flagger;

	if ( Message == class'Botpack.CTFMessage' )
	{
		switch (Switch)
		{
			case 0: // Capped CTFFlag(OptionalObject) by RelatedPRI_1
				if ( bShowFlagCountDown && CTFFlag(OptionalObject) != None )
				{
					SetFlagTimer(CTFFlag(OptionalObject).Team, False);
				}
				if (bShowFlagCarrierName && CTFFlag(OptionalObject) != None )
				{
					FlagCarrier[CTFFlag(OptionalObject).Team] = None;
				}
			break;

			case 1: // Returned CTFFlag(OptionalObject) by RelatedPRI_1
				if ( bShowFlagCountDown && CTFFlag(OptionalObject) != None )
				{
					SetFlagTimer(CTFFlag(OptionalObject).Team, False);
				}
			break;
		
			case 2: // Dropped by RelatedPRI_1, TeamInfo(OptionalObject)
				if ( bShowFlagCountDown && TeamInfo(OptionalObject) != None  )
				{
					SetFlagTimer(TeamInfo(OptionalObject).TeamIndex, True);
				}
				if (bShowFlagCarrierName && TeamInfo(OptionalObject) != None )
				{
					FlagCarrier[TeamInfo(OptionalObject).TeamIndex] = None;
				}
			break;

			case 3: // Flag was Auto returned, TeamInfo(OptionalObject)
				if (bShowFlagCountDown && TeamInfo(OptionalObject) != None )
				{
					SetFlagTimer(TeamInfo(OptionalObject).TeamIndex, False);
				}
			break;

			case 4: // Flag was dropped and picked up by RelatedPRI_1, TeamInfo(OptionalObject)
				if (bShowFlagCountDown && TeamInfo(OptionalObject) != None )
				{
					SetFlagTimer(TeamInfo(OptionalObject).TeamIndex, False);
				}
				if (bShowFlagCarrierName && TeamInfo(OptionalObject) != None )
				{
					FlagCarrier[TeamInfo(OptionalObject).TeamIndex] = RelatedPRI_1;
				}
			break;

			case 5: // Auto return, TeamInfo(OptionalObject)
				if (bShowFlagCountDown && TeamInfo(OptionalObject) != None )
				{
					SetFlagTimer(TeamInfo(OptionalObject).TeamIndex, False);
				}
			break;

			case 6: // Flag was home and picked up by RelatedPRI_1, TeamInfo(OptionalObject)
				if (bShowFlagCountDown && TeamInfo(OptionalObject) != None )
				{
					SetFlagTimer(TeamInfo(OptionalObject).TeamIndex, False);
				}
				if (bShowFlagCarrierName && TeamInfo(OptionalObject) != None )
				{
					FlagCarrier[TeamInfo(OptionalObject).TeamIndex] = RelatedPRI_1;
				}
			break;
		}

	}
	// Allow other mutators to work...
	if ( NextMessageMutator != None )
		return NextMessageMutator.MutatorBroadcastLocalizedMessage( Sender, Receiver, Message, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject );
	else
		return true;
}

function bool HandlePickupQuery(Pawn Other, Inventory item, out byte bAllowPickup)
{// Used to prevent player's from picking up a shieldbelt if their current one is already full
// return true means to prevent the pickup
	local inventory I, BrGun;
	local bool bItemFound;

	bItemFound = false;
	if ( Item != None )
	{
		for ( I=Other.Inventory; I!=None; I=I.Inventory )
		{
			if ( bPreventArmorPickupWhenFull && Item.bIsAnArmor && I.bIsAnArmor && Item.Class == I.Class && I.Charge >= I.Default.Charge )
			{
				bItemFound = true;
			}
			else if ( !bAlwaysPreventMultiBeltPickup && Item.IsA('UT_ShieldBelt') && I.IsA('UT_ShieldBelt') && I.Charge < I.Default.Charge )
			{
				bItemFound = false;
			}
			else if ( bPreventMultiBootsWhenFull && Item.IsA('UT_JumpBoots') && I.IsA('UT_JumpBoots') && I.Charge >= I.Default.Charge )
			{
				bItemFound = true;
			}
			else if ( bPreventMultiDamAmp && (( Item.IsA('UDamage') && I.IsA('UDamage') ) || ( Item.IsA('Amplifier') && I.IsA('Amplifier') )) )
			{
				bItemFound = true;
			}
			else if ( bPreventMultiInvis && Item.IsA('UT_Invisibility') && I.IsA('UT_Invisibility') )
			{
				bItemFound = true;
			}
			else if ( I.IsA('UT_ShieldBelt') && Item.bIsAnArmor )
			{
				if ( I.Charge >= I.Default.Charge )
				{// Belt is fulll
					if ( bPreventArmorPickupWhenHasFullBelt )
					{// prevent pickup
						bItemFound = true;
					}
				}
				else if ( bPreventArmorPickupWhenHasDamagedBelt )
				{// prevent pickup
					bItemFound = true;
				}
			}
			else if ( bPreventMultiScubaGear && Item.IsA('ScubaGear') && I.IsA('ScubaGear') )
			{
				bItemFound = true;
			}
			else if ( bPreventMultiFlashLight && Item.IsA('Flashlight') && !Item.IsA('SearchLight') && I.IsA('FlashLight') && !I.IsA('SearchLight') )
			{
				bItemFound = true;
			}
			else if ( bPreventMultiSearchLight && Item.IsA('Searchlight') && I.IsA('SearchLight') )
			{
				bItemFound = true;
			}
			else if ( bPreventMultiSuits && Item.IsA('Suits') && I.IsA('Suits') && I.Class == Item.Class )
			{
				bItemFound = true;
			}
			else if ( Item.IsA('Armor') && I.IsA('Armor') )
			{
				bItemFound = true;
			}
			else if ( !Item.IsA('Ammo') && bPreventAllMultiPickups && Pickup(Item) != None && Pickup(I) != None && I.Class == Item.Class )
			{// cover any item not already included
				bItemFound = true;
			}
		}
		if ( bItemFound )
		{// found item in our inventory, so prevent pickup
			/*if( Other.IsA('Bot') && bEnableBotSupportOnPickups )
			{// make bots fear this item
				Bot(Other).FearThisSpot(item);
			}*/
			return true;
		}
	}
	else
	{
		return false;
	}

	if ( NextMutator != None )
		return NextMutator.HandlePickupQuery(Other, item, bAllowPickup);
	return false;
}
function SetFlagTimer( int Team, bool bActive )
{// Keeps track of what flags are out by enabling/disabling the countdown timers
	if ( Team == 0 )
	{// Red
		if ( bActive )
		{
			FlagRed.SetTimer(1,True);
			CountdownR = 25;
			bRedOut = True;
		}
		else
		{
			FlagRed.SetTimer(0,False);
			CountdownR = 25;
			bRedOut = False;
		}
	}
	else if ( Team == 1 )
	{// Blue
		if ( bActive )
		{
			FlagBlue.SetTimer(1,True);
			CountdownB = 25;
			bBlueOut = True;
		}
		else
		{
			FlagBlue.SetTimer(0,False);
			CountdownB = 25;
			bBlueOut = False;
		}
	}
}

defaultproperties
{
     bShowFlagCountDown=True
     bShowFlagCarrierName=True
     bShowAmpCountdown=True
     bShowInvisCountdown=True
     bShowBootCharge=True
     bPreventArmorPickupWhenFull=True
     bAlwaysPreventMultiBeltPickup=True
     bPreventMultiDamAmp=True
     bPreventMultiBootsWhenFull=True
     bPreventMultiInvis=True
     bPreventArmorPickupWhenHasFullBelt=True
     bPreventMultiScubaGear=True
     bPreventMultiFlashLight=True
     bPreventMultiSearchLight=True
     bPreventMultiSuits=True
     bPreventAllMultiPickups=True
     bModifyTriggers=True
     bShowBombingRunCarrier=True
     MapToModify(0)="CTF-2ChickenPancakes"
     ModifiedTriggerEvent(0)=skyfall
     NewReTriggerDelay(0)=65
     bAlwaysRelevant=True
     RemoteRole=ROLE_SimulatedProxy
}
